// main.c -- Defines the C-code kernel entry point, calls initialisation routines.
//           Made for JamesM's tutorials <www.jamesmolloy.co.uk>

// Casnix OS kernel casnuz.bin 3.1.1.2.01
#include <monitor.h>
#include <descriptor_tables.h>
#include <timer.h>
#include <paging.h>
#include <multiboot.h>
#include <fs.h>
#include <initrd.h>
#include <task.h>
#include <syscall.h>

extern void install_kb();
int __debug = 1;
extern u32int placement_address;
u32int initial_esp;
u8int backColour, foreColour;
int main(struct multiboot *mboot_ptr, u32int initial_stack)
{
    initial_esp = initial_stack;
    // Initialise all the ISRs and segmentation
    init_descriptor_tables();
    // Initialise the screen (by clearing it)
    backColour = 9;
    foreColour = 11;
    clear_screen();
	puts("In zone 0.\n");
    // Initialise the PIT to 100Hz
    if (__debug == 1) puts("Starting timer...\n");
    asm volatile("sti");
    init_timer(50);
	puts("Installing keyboard...\n");
	outb(0x64, 0xf5);
	outb(0x64, 0xff);
	outb(0x64, 0xf4);
	install_kb();
    // Find the location of our initial ramdisk.
    ASSERT(mboot_ptr->mods_count > 0);
    u32int initrd_location = *((u32int*)mboot_ptr->mods_addr);
    u32int initrd_end = *(u32int*)(mboot_ptr->mods_addr+4);
    // Don't trample our module with placement accesses, please!
    placement_address = initrd_end;
	puts("In zone 1.\n");
    // Start paging.
    initialise_paging();
	
    // Start multitasking.
    initialise_tasking();
	puts("In zone 2.\n");
    // Initialise the initial ramdisk, and set it as the filesystem root.
    fs_root = initialise_initrd(initrd_location);

	
	
    initialise_syscalls();
	puts("In zone 3.\n");
	puts("Casnix OS 3 Alpha 1.2_00 booted.\n");
	puts("casnuz is running.\n");
	puts("I was started with the following command line:\n");
	puts((char*)mboot_ptr->cmdline);
	puts("\n\nCasnix OS 3 (c) Matthew Caswell\n");
	puts("Creating exceptions...\n");
	foreColour = 12;
	char b[128];
	gets(b);
	puts("Switching to user mode...\n");
    backColour= 15;
    foreColour= 12;
   	switch_to_user_mode();

  	kputs("Switched to zone 4.  System calls enabled.\n");
	kset_colors(11, 9);
	kputs("Casnix OS 3. Casnix OS kernel casnuz.bin 3.1.1.1.16\n");
	kputs("\nStarted.\n");
	kputs("\n\n[CASNUZ] Boot$ ");
	char buf[128];
	kgets(buf);
	if (buf[0] == "v" && buf[1] == "e" && buf[2] == "r") {
		kputs("\n\nCasnix OS 3 Alpha 1.2\ncasnuz 3.1.1.2.01\n");
	} else {
		kputs("\nInvalid command -- try 'ver'\n");
	}
	
    return 0;
}
