// syscall.c -- Defines the implementation of a system call system.
//              Written for JamesM's kernel development tutorials.

#include <syscall.h>

#include <monitor.h>
#include <kb.h>

static void syscall_handler(registers_t *regs);

DEFN_SYSCALL1(puts, 0, const char*);
DEFN_SYSCALL1(put_hex, 1, const char*);
DEFN_SYSCALL1(put_dec, 2, const char*);
DEFN_SYSCALL2(set_colors, 3, u8int, u8int);
DEFN_SYSCALL1(putc, 4, const char);
DEFN_SYSCALL0(clear_screen, 5);
DEFN_SYSCALL2(move_cursor, 6, int, int);
DEFN_SYSCALL1(gets, 7, char*);

	
static void *syscalls[MAX_SYSCALLS] =
{
    &puts,
    &put_hex,
    &put_dec,
    &set_colors,
    &putc,
    &clear_screen,
    &move_cursor,
    &gets,
};
u32int num_syscalls = 8;

void initialise_syscalls()
{
    // Register our syscall handler.
    register_interrupt_handler (90, &syscall_handler);
}

void syscall_handler(registers_t *regs)
{
    // Firstly, check if the requested syscall number is valid.
    // The syscall number is found in EAX.
    if (regs->eax >= num_syscalls)
        return;

    // Get the required syscall location.
    void *location = syscalls[regs->eax];

    // We don't know how many parameters the function wants, so we just
    // push them all onto the stack in the correct order. The function will
    // use all the parameters it wants, and we can pop them all back off afterwards.
    int ret;
    asm volatile (" \
      push %1; \
      push %2; \
      push %3; \
      push %4; \
      push %5; \
      call *%6; \
      pop %%ebx; \
      pop %%ebx; \
      pop %%ebx; \
      pop %%ebx; \
      pop %%ebx; \
    " : "=a" (ret) : "r" (regs->edi), "r" (regs->esi), "r" (regs->edx), "r" (regs->ecx), "r" (regs->ebx), "r" (location));
    regs->eax = ret;
}
